/* Programmname                                                               */

/*----------------------------------------------------------------------------*/
/* System Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/intuition.h>
	#include <proto/utility.h>
	#include <clib/alib_protos.h>
	#include <reaction/reaction.h>
	#include <reaction/reaction_macros.h>
	#include <classes/window.h>
	#include <proto/window.h>
	#include <proto/layout.h>
	#include <proto/button.h>
	#include <proto/string.h>
	#include <proto/listbrowser.h>
	#include <proto/getfile.h>
	#include <gadgets/getfile.h>
	#include <stdlib.h>
	#include <string.h>

/*----------------------------------------------------------------------------*/
/* Konstanten & Makros                                                        */
/*----------------------------------------------------------------------------*/

	#define GID_LIST        1001
	#define GID_STRING      1002
	#define GID_ADD         1003
	#define GID_REMOVE      1004
	#define GID_SORT        1005
	#define GID_LOAD        1006
	#define GID_SAVE        1007
	#define GID_QUIT        1008

	#ifdef __amigaos4__
	 #define BASEDEF(base)      struct Library *base##Base = NULL; struct base##IFace *I##base = NULL
	 #define OPENLIB(base,name) if (!((base##Base) = OpenLibrary ((name),0)) || !((I##base) = (void *)GetInterface ((base##Base),"main",1,NULL))) ok = FALSE
	 #define CLOSELIB(base)     if (I##base) DropInterface ((struct Interface *)(I##base)); if (base##Base) CloseLibrary (base##Base)
	#else
	 #define BASEDEF(base)      struct Library *base##Base = NULL
	 #define OPENLIB(base,name) if (!((base##Base) = OpenLibrary ((name),0))) ok = FALSE
	 #define CLOSELIB(base)     if (base##Base) CloseLibrary (base##Base)

	 #define NewList(list) ((list)->lh_Head = (struct Node *)&(list)->lh_Tail,(list)->lh_TailPred = (struct Node *)&(list)->lh_Head,(list)->lh_Tail = NULL)
	 #define GetHead(list) ((list) && (list)->lh_Head && (list)->lh_Head->ln_Succ ? (list)->lh_Head : NULL)
	 #define GetTail(list) ((list) && (list)->lh_TailPred && (list)->lh_TailPred->ln_Pred ? (list)->lh_TailPred : NULL)
	 #define GetSucc(node) ((node) && (node)->ln_Succ->ln_Succ ? (node)->ln_Succ : NULL)
	 #define GetPred(node) ((node) && (node)->ln_Pred->ln_Pred ? (node)->ln_Pred : NULL)
	#endif

/*----------------------------------------------------------------------------*/
/* Typdefinitionen                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Globale Variablen                                                          */
/*----------------------------------------------------------------------------*/

	BASEDEF(Window);
	BASEDEF(Layout);
	BASEDEF(Button);
	BASEDEF(String);
	BASEDEF(ListBrowser);
	#undef GetFile
	BASEDEF(GetFile);

/*----------------------------------------------------------------------------*/
/* ReAction-Klassen ffnen                                                    */
/*----------------------------------------------------------------------------*/

	BOOL open_classes (void)

	{
	BOOL ok = TRUE;

	OPENLIB(Window,"window.class");
	OPENLIB(Layout,"gadgets/layout.gadget");
	OPENLIB(Button,"gadgets/button.gadget");
	OPENLIB(String,"gadgets/string.gadget");
	OPENLIB(ListBrowser,"gadgets/listbrowser.gadget");
	OPENLIB(GetFile,"gadgets/getfile.gadget");

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/* ReAction-Klassen schlieen                                                 */
/*----------------------------------------------------------------------------*/

	void close_classes (void)

	{
	CLOSELIB(Window);
	CLOSELIB(Layout);
	CLOSELIB(Button);
	CLOSELIB(String);
	CLOSELIB(ListBrowser);
	CLOSELIB(GetFile);
	}

/*----------------------------------------------------------------------------*/
/* Listbrowser-Liste aus Datei laden                                          */
/*----------------------------------------------------------------------------*/

	BOOL load_list (char *name,struct List *list)

	{
	BPTR file;
	BOOL ok = FALSE;
	char line[256];
	struct Node *node;
	long l;

	if (file = Open (name,MODE_OLDFILE))
		{
		while (FGets (file,line,256))
			{
			if (l = strlen(line))
				{
				l --;
				if (line[l] == '\n')
					line[l] = 0;
				}
			if (node = AllocListBrowserNode (1,LBNCA_CopyText,TRUE,LBNCA_Text,line,TAG_END))
				AddTail (list,node);
			}
		ok = TRUE;
		Close (file);
		}

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/* Listbrowser-Liste in Datei speichern                                       */
/*----------------------------------------------------------------------------*/

	BOOL save_list (char *name,struct List *list)

	{
	BPTR file;
	BOOL ok = FALSE;
	char *line;
	struct Node *node;

	if (file = Open (name,MODE_NEWFILE))
		{
		for (node = GetHead (list); node; node = GetSucc (node))
			{
			GetListBrowserNodeAttrs (node,LBNCA_Text,&line,TAG_END);
			FPuts (file,line);
			FPutC (file,'\n');
			}
		ok = TRUE;
		Close (file);
		}

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/* Zwei Listbrowser-Nodes vergleichen (fr Sort)                              */
/*----------------------------------------------------------------------------*/

	int compare_lbnodes (struct Node **n1,struct Node **n2)

	{
	char *text1,*text2;
	GetListBrowserNodeAttrs (*n1,LBNCA_Text,&text1,TAG_END);
	GetListBrowserNodeAttrs (*n2,LBNCA_Text,&text2,TAG_END);
	return (Stricmp (text1,text2));
	}

/*----------------------------------------------------------------------------*/
/* Listbrowser-Liste sortieren                                                */
/*----------------------------------------------------------------------------*/

	void sort_list (struct List *list)

	{
	struct Node *node;
	struct Node **array;
	long n,i;

	n = 0;
	for (node = GetHead (list); node; node = GetSucc (node))
		n ++;

	if (array = AllocVec (n * sizeof(struct Node *),0))
		{
		for (i = 0; i < n; i++)
			array[i] = RemHead (list);

		qsort (array,n,sizeof(struct Node *),(void *)compare_lbnodes);

		for (i = 0; i < n; i++)
			AddTail (list,array[i]);

		FreeVec (array);
		}
	}

/*----------------------------------------------------------------------------*/
/* Hauptprogramm                                                              */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	Object *winobj;
	struct Screen *scr;
	struct Window *win;
	ULONG winsig;
	ULONG sigs;
	BOOL cont;
	ULONG result;
	UWORD code;
	struct Gadget *root;
	struct Gadget *lbo;
	struct Gadget *sto;
	Object *gfo;
	struct List lblist;
	struct Node *node = NULL;
	struct Node *nextnode;
	char *text;

	if (!open_classes())
		{
		Printf ("cannot load required classes\n");
		close_classes();
		return (20);
		}

	NewList (&lblist);

	if (scr = LockPubScreen (NULL))
		{
		gfo = GetFileObject, EndObject;

		winobj = WindowObject,
			WA_Title,"My first ReAction program",
			WA_Width,scr->Width  * 4 / 10,
			WA_Height,scr->Height * 6 / 10,
			WA_Flags,WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_SIZEGADGET|WFLG_SIZEBBOTTOM|WFLG_ACTIVATE,
			WA_PubScreen,scr,
			WINDOW_Position,WPOS_CENTERSCREEN,
			WINDOW_Layout,root = VGroupObject,
				LAYOUT_SpaceOuter,TRUE,
				LAYOUT_AddChild,lbo = ListBrowserObject,
					GA_ID,GID_LIST,
					GA_RelVerify,TRUE,
					LISTBROWSER_Labels,&lblist,
					LISTBROWSER_ShowSelected,TRUE,
				EndObject,
				LAYOUT_AddChild,sto = StringObject,
					GA_ID,GID_STRING,
					GA_RelVerify,TRUE,
					GA_Disabled,TRUE,
				EndObject,
				StartHGroup,
					LAYOUT_EvenSize,TRUE,
					LAYOUT_FixedVert,FALSE,
					LAYOUT_AddChild,
						Button("Add",GID_ADD),
					LAYOUT_AddChild,
						Button("Remove",GID_REMOVE),
					LAYOUT_AddChild,
						Button("Sort",GID_SORT),
				EndHGroup,
				StartHGroup,
					LAYOUT_EvenSize,TRUE,
					LAYOUT_FixedVert,FALSE,
					LAYOUT_AddChild,
						Button("Load",GID_LOAD),
					LAYOUT_AddChild,
						Button("Save",GID_SAVE),
					LAYOUT_AddChild,
						Button("Quit",GID_QUIT),
				EndHGroup,
			EndVGroup,
		EndWindow;

		if (gfo && winobj)
			{
			if (win = RA_OpenWindow (winobj))
				{
				GetAttr (WINDOW_SigMask,winobj,&winsig);

				cont = TRUE;
				while (cont)
					{
					sigs = Wait (winsig | SIGBREAKF_CTRL_C);

					if (sigs & SIGBREAKF_CTRL_C)
						cont = FALSE;

					if (sigs & winsig)
						{
						while ((result = RA_HandleInput(winobj,&code)) != WMHI_LASTMSG)
							{
							switch(result & WMHI_CLASSMASK)
								{
							case WMHI_GADGETUP:
								switch(result & WMHI_GADGETMASK)
									{
								case GID_LOAD:
									SetAttrs (gfo,GETFILE_DoSaveMode,FALSE,TAG_END);
									if (gfRequestFile (gfo,win))
										{
										GetAttr (GETFILE_FullFile,gfo,(ULONG *)&text);
										SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,-1,TAG_END);
										FreeListBrowserList (&lblist);
										load_list (text,&lblist);
										SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,&lblist,LISTBROWSER_SelectedNode,NULL,TAG_END);
										SetGadgetAttrs (sto,win,NULL,STRINGA_TextVal,"",GA_Disabled,TRUE,TAG_END);
										node = NULL;
										}
									break;
								case GID_SAVE:
									SetAttrs (gfo,GETFILE_DoSaveMode,TRUE,TAG_END);
									if (gfRequestFile (gfo,win))
										{
										GetAttr (GETFILE_FullFile,gfo,(ULONG *)&text);
										save_list (text,&lblist);
										}
									break;
								case GID_SORT:
									SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,-1,TAG_END);
									sort_list (&lblist);
									SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,&lblist,LISTBROWSER_SelectedNode,NULL,TAG_END);
									SetGadgetAttrs (sto,win,NULL,STRINGA_TextVal,"",GA_Disabled,TRUE,TAG_END);
									node = NULL;
									break;
								case GID_QUIT:
									cont = FALSE;
									break;
								case GID_LIST:
									GetAttr (LISTBROWSER_SelectedNode,lbo,(ULONG *)&node);
									if (node)
										{
										GetListBrowserNodeAttrs (node,LBNCA_Text,&text,TAG_END);
										SetGadgetAttrs (sto,win,NULL,STRINGA_TextVal,text,GA_Disabled,FALSE,TAG_END);
										ActivateLayoutGadget (root,win,NULL,(ULONG)sto);
										}
									break;
								case GID_STRING:
									if (node)
										{
										GetAttr (STRINGA_TextVal,sto,(ULONG *)&text);
										SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,-1,TAG_END);
										SetListBrowserNodeAttrs (node,LBNCA_CopyText,TRUE,LBNCA_Text,text,TAG_END);
										SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,&lblist,LISTBROWSER_SelectedNode,node,TAG_END);
										}
									break;
								case GID_REMOVE:
									if (node)
										{
										SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,-1,TAG_END);
										nextnode = GetSucc (node);
										Remove (node);
										FreeListBrowserNode (node);
										node = nextnode;
										if (node)
											{
											GetListBrowserNodeAttrs (node,LBNCA_Text,&text,TAG_END);
											SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,&lblist,LISTBROWSER_SelectedNode,node,TAG_END);
											SetGadgetAttrs (sto,win,NULL,STRINGA_TextVal,text,GA_Disabled,FALSE,TAG_END);
											ActivateLayoutGadget (root,win,NULL,(ULONG)sto);
											}
										else
											{
											SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,&lblist,LISTBROWSER_SelectedNode,NULL,TAG_END);
											SetGadgetAttrs (sto,win,NULL,STRINGA_TextVal,"",GA_Disabled,TRUE,TAG_END);
											}
										}
									break;
								case GID_ADD:
									nextnode = node;
									if (node = AllocListBrowserNode (1,TAG_END))
										{
										SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,-1,TAG_END);
										if (nextnode)
											Insert (&lblist,node,nextnode);
										else
											AddTail (&lblist,node);
										SetGadgetAttrs (lbo,win,NULL,LISTBROWSER_Labels,&lblist,LISTBROWSER_SelectedNode,node,TAG_END);
										SetGadgetAttrs (sto,win,NULL,STRINGA_TextVal,"",GA_Disabled,FALSE,TAG_END);
										ActivateLayoutGadget (root,win,NULL,(ULONG)sto);
										}
									break;
									}
								break;
							case WMHI_CLOSEWINDOW:
								cont = FALSE;
								break;
								}
							}
						}
					}
				}
			else
				Printf ("cannot open window\n");
			}
		else
			Printf ("cannot create objects\n");

		if (winobj)
			DisposeObject (winobj);
		if (gfo)
			DisposeObject (gfo);

		UnlockPubScreen (NULL,scr);
		}

	FreeListBrowserList (&lblist);

	close_classes();
	return (0);
	}

/*----------------------------------------------------------------------------*/
/* Ende des Quelltextes                                                       */
/*----------------------------------------------------------------------------*/
